@@ -4,7 +4,9 @@ import codecs
 import configparser
 import datetime
 import subprocess
+import string
 from contextlib import contextmanager
+import unidecode
 import jinja2
 from python_boilerplate.inputs import *
 
@@ -16,10 +18,36 @@ license_alias = {
 }
 
 
+def pyname(name):
+    """
+    Converts a string of text into a valid python name.
+    """
+
+    name = unidecode.unidecode(name.strip().lower())
+    valid = string.ascii_letters + string.digits + '_'
+    char_list = []
+    for i, char in enumerate(name):
+        if char in valid:
+            char_list.append(char)
+        elif char in '- \t\n':
+            char_list.append('_')
+
+    # Join characters and take precautions against weird names
+    name = ''.join(char_list)
+    if not name:
+        return 'project'
+    elif name[0].isdigit():
+        return 'py' + name
+    else:
+        return name
+
+
 def write_template(template: str, namespace=None, ignore=False, path=None,
                    verbose=True, hash=None):
-    """Render jinja template with the given namespace and saves it in the
-    desired path"""
+    """
+    Render jinja template with the given namespace and saves it in the
+    desired path
+    """
 
     if path is None:
         path = template
@@ -31,16 +59,16 @@ def write_template(template: str, namespace=None, ignore=False, path=None,
         return
     elif os.path.exists(path):
         with open(path) as F:
-            filedata = F.read()
-            if filedata == data:
+            file_data = F.read()
+            if file_data == data:
                 return
 
         # If hash is compatible with given hash, we simply overwrite
         ask = True
         if hash:
-            filehash = hashlib.md5(filedata.encode('utf8')).digest()
-            filehash = codecs.encode(filehash, 'base64').decode()
-            if hash == filehash:
+            file_hash = hashlib.md5(file_data.encode('utf8')).digest()
+            file_hash = codecs.encode(file_hash, 'base64').decode()
+            if hash == file_hash:
                 os.rename(path, path + '.bak')
                 ask = False
 
@@ -62,13 +90,13 @@ def write_template(template: str, namespace=None, ignore=False, path=None,
 
 
 @contextmanager
-def visit_dir(dir):
+def visit_dir(path):
     """Visit directory and come back after the with block is finish."""
 
-    currdir = os.getcwd()
-    os.chdir(dir)
+    current_dir = os.getcwd()
+    os.chdir(path)
     yield
-    os.chdir(currdir)
+    os.chdir(current_dir)
 
 
 class BoilerplateConfig(configparser.ConfigParser):
@@ -95,9 +123,9 @@ class BoilerplateConfig(configparser.ConfigParser):
         """Compute and register the hash value for some file given its textual
         data."""
 
-        filehash = hashlib.md5(data.encode('utf8')).digest()
-        filehash = codecs.encode(filehash, 'base64').decode()
-        self.set('file hashes', filename, filehash)
+        file_hash = hashlib.md5(data.encode('utf8')).digest()
+        file_hash = codecs.encode(file_hash, 'base64').decode()
+        self.set('file hashes', filename, file_hash)
 
     def save(self):
         """Saves content to boilerplate.ini."""
@@ -127,6 +155,9 @@ class ProjectFactory:
         if kwds:
             raise TypeError('invalid attribute: %s' % kwds.popitem()[0])
 
+        # This is the project name as a valid python name
+        self.pyname = self.project and pyname(self.project)
+
     def write_template(self, template, ignore=False, path=None):
         base = os.getcwd()
         file = os.path.join(base, path or template)
@@ -139,6 +170,7 @@ class ProjectFactory:
         del namespace['basepath_len']
         namespace['job'] = self
         namespace['date'] = datetime.datetime.now()
+        namespace['dashed_pyname'] = self.pyname.replace('_', '-')
 
         # Write data and fetch hash
         data = write_template(template, namespace, ignore=ignore, path=path)
@@ -154,7 +186,7 @@ class ProjectFactory:
     #
     # New project and new sections
     #
-    def new_project(self, ask_editor=True):
+    def new_project(self, ask_editor=None):
         # Asks basic info
         if self.project is None:
             self.project = input("Project's name: ")
@@ -162,6 +194,8 @@ class ProjectFactory:
             self.author = input("Author's name: ")
         if self.email is None:
             self.email = input("Author's email: ")
+        if not self.pyname:
+            self.pyname = default_input("Python name: ", pyname(self.project))
 
         # Fetch version from existing VERSION file or asks the user
         if self.version is None and os.path.exists('VERSION'):
@@ -172,6 +206,9 @@ class ProjectFactory:
         self.license = self.license or default_input('License: ', 'gpl')
 
         # Save config file
+        if ask_editor is None:
+            ask_editor = not os.path.exists('boilerplate.ini')
+
         self.save_config()
         if ask_editor:
             print('Your config file was saved as boilerplate.ini. You can '
@@ -190,29 +227,30 @@ class ProjectFactory:
         self.make_docs(ignore=False)
         self.make_license(ignore=False)
         self.make_invoke(ignore=False)
+        self.save_config()
 
     def make_base(self, ignore=True):
         self.write_template('VERSION.txt', ignore=True, path='VERSION')
         self.write_template('gitignore.txt', ignore=ignore, path='.gitignore')
 
     def make_setup_py(self, ignore=True):
-        self.write_template('setup.py', ignore=ignore)
+        self.write_template('setup.pyt', ignore=ignore, path='setup.py')
         self.write_template('MANIFEST.in', ignore=ignore)
         self.write_template('requirements.txt', ignore=ignore)
         self.write_template('requirements-dev.txt', ignore=ignore)
 
     def make_package(self, ignore=True):
-        curdir = os.getcwd()
+        current_dir = os.getcwd()
         try:
             # Make src/project directory
-            for directory in ['src', self.project]:
+            for directory in ['src', self.pyname]:
                 if not os.path.exists(directory):
                     os.mkdir(directory)
                 os.chdir(directory)
 
-            self.write_template('package/init.py', True, '__init__.py')
-            self.write_template('package/__main__.py', ignore, '__main__.py')
-            self.write_template('package/__meta__.py', True, '__meta__.py')
+            self.write_template('package/init.pyt', True, '__init__.py')
+            self.write_template('package/main.pyt', ignore, '__main__.py')
+            self.write_template('package/meta.pyt', True, '__meta__.py')
 
             # Make test folder
             if not os.path.exists('tests'):
@@ -223,10 +261,10 @@ class ProjectFactory:
                 with open('__init__.py', 'w') as F:
                     F.write('\n')
             self.write_template(
-                'package/test_project.py', ignore, 'test_%s.py' % self.project)
+                'package/test_project.pyt', ignore, 'test_%s.py' % self.pyname)
 
         finally:
-            os.chdir(curdir)
+            os.chdir(current_dir)
 
     def make_readme(self, ignore=True):
         self.write_template('README.rst', ignore=True)
@@ -238,7 +276,7 @@ class ProjectFactory:
         # Make src/project directory
         if not os.path.exists('docs'):
             os.mkdir('docs')
-        self.write_template('docs/conf.py', ignore=ignore)
+        self.write_template('docs/conf.pyt', ignore=ignore, path='docs/conf.py')
         self.write_template('docs/index.rst', ignore=ignore)
         self.write_template('docs/install.rst', ignore=ignore)
         self.write_template('docs/license.rst', ignore=ignore)
@@ -281,7 +319,7 @@ class ProjectFactory:
         pass
 
     def make_invoke(self, ignore=True):
-        self.write_template('tasks.py', ignore)
+        self.write_template('tasks.pyt', ignore=ignore, path='tasks.py')
 
     def save_config(self):
         for attr in BoilerplateConfig.valid_attrs:
@@ -290,8 +328,10 @@ class ProjectFactory:
                 self.config.remove_option('options', attr)
             else:
                 self.config.set('options', attr, value)
+        self.config.save()
 
 
+# noinspection PyPep8Naming
 class attrdict(dict):
     """Dictionary with attribute access to keys."""
 
